@@ -1,5 +1,5 @@
 /* String-list functions for GNU Mailutils.
-   Copyright (C) 2007-2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2007-2012, 2014-2017 Free Software Foundation, Inc.
 
    Based on slist module from GNU Radius.  Written by Sergey Poznyakoff.
    
@@ -22,6 +22,7 @@
 #endif
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 #include <mailutils/types.h>
 #include <mailutils/alloc.h>
@@ -31,38 +32,49 @@
 #include <mailutils/nls.h>
 #include <mailutils/iterator.h>
 
-struct mu_opool_bucket
+struct bucket_header
 {
-  struct mu_opool_bucket *next;
+  union mu_opool_bucket *next;
   char *buf;
   size_t level;
   size_t size;
 };
 
-struct _mu_opool
+union mu_opool_bucket
 {
-  int memerr;
-  size_t bucket_size;
-  size_t itr_count;
-  struct mu_opool_bucket *head, *tail;
-  struct mu_opool_bucket *free;
+  struct bucket_header hdr;
+  long double align_double;
+  uintmax_t align_uintmax;
+  void *align_ptr;
 };
 
-static struct mu_opool_bucket *
+struct _mu_opool
+{
+  int flags;                   /* Flag bits */
+  size_t bucket_size;          /* Default bucket size */
+  size_t itr_count;            /* Number of iterators created for this pool */
+  mu_nonlocal_jmp_t *jmp;      /* Buffer for non-local exit */
+  union mu_opool_bucket *bkt_head, *bkt_tail; 
+  union mu_opool_bucket *bkt_fini; /* List of finished objects */
+};
+
+static union mu_opool_bucket *
 alloc_bucket (struct _mu_opool *opool, size_t size)
 {
-  struct mu_opool_bucket *p = malloc (sizeof (*p) + size);
+  union mu_opool_bucket *p = malloc (sizeof (*p) + size);
   if (!p)
     {
-      if (opool->memerr)
+      if (opool->flags & MU_OPOOL_ENOMEMABRT)
 	mu_alloc_die ();
+      if (opool->jmp)
+	longjmp (opool->jmp->buf, ENOMEM);
     }
   else
     {
-      p->buf = (char*)(p + 1);
-      p->level = 0;
-      p->size = size;
-      p->next = NULL;
+      p->hdr.buf = (char*)(p + 1);
+      p->hdr.level = 0;
+      p->hdr.size = size;
+      p->hdr.next = NULL;
     }
   return p;
 }
@@ -70,14 +82,14 @@ alloc_bucket (struct _mu_opool *opool, size_t size)
 static int
 alloc_pool (mu_opool_t opool, size_t size)
 {
-  struct mu_opool_bucket *p = alloc_bucket (opool, opool->bucket_size);
+  union mu_opool_bucket *p = alloc_bucket (opool, opool->bucket_size);
   if (!p)
     return ENOMEM;
-  if (opool->tail)
-    opool->tail->next = p;
+  if (opool->bkt_tail)
+    opool->bkt_tail->hdr.next = p;
   else
-    opool->head = p;
-  opool->tail = p;
+    opool->bkt_head = p;
+  opool->bkt_tail = p;
   return 0;
 }
 
@@ -86,36 +98,57 @@ copy_chars (mu_opool_t opool, const char *str, size_t n, size_t *psize)
 {
   size_t rest;
 
-  if (!opool->head || opool->tail->level == opool->tail->size)
+  if (!opool->bkt_head
+      || opool->bkt_tail->hdr.level == opool->bkt_tail->hdr.size)
     if (alloc_pool (opool, opool->bucket_size))
       return ENOMEM;
-  rest = opool->tail->size - opool->tail->level;
+  rest = opool->bkt_tail->hdr.size - opool->bkt_tail->hdr.level;
   if (n > rest)
     n = rest;
-  memcpy (opool->tail->buf + opool->tail->level, str, n);
-  opool->tail->level += n;
+  memcpy (opool->bkt_tail->hdr.buf + opool->bkt_tail->hdr.level, str, n);
+  opool->bkt_tail->hdr.level += n;
   *psize = n;
   return 0;
 }
 
 int
-mu_opool_create (mu_opool_t *pret, int memerr)
+mu_opool_create (mu_opool_t *pret, int flags)
 {
   struct _mu_opool *x = malloc (sizeof (x[0]));
   if (!x)
     {
-      if (memerr)
+      if (flags & MU_OPOOL_ENOMEMABRT)
 	mu_alloc_die ();
       return ENOMEM;
     }
-  x->memerr = memerr;
+  x->flags = flags;
   x->bucket_size = MU_OPOOL_BUCKET_SIZE;
   x->itr_count = 0;
-  x->head = x->tail = x->free = 0;
+  x->bkt_head = x->bkt_tail = x->bkt_fini = NULL;
+  x->jmp = NULL;
   *pret = x;
   return 0;
 }
 
+void
+mu_opool_setjmp (mu_opool_t opool, mu_nonlocal_jmp_t *jmp)
+{
+  if (jmp)
+    {
+      jmp->next = opool->jmp;
+      opool->jmp = jmp;
+    }
+  else
+    mu_opool_clrjmp (opool);
+}
+
+void
+mu_opool_clrjmp (mu_opool_t opool)
+{
+  if (opool->jmp)
+    opool->jmp = opool->jmp->next;
+}
+
 int
 mu_opool_set_bucket_size (mu_opool_t opool, size_t size)
 {
@@ -140,31 +173,31 @@ mu_opool_clear (mu_opool_t opool)
   if (!opool)
     return;
   
-  if (opool->tail)
+  if (opool->bkt_tail)
     {
-      opool->tail->next = opool->free;
-      opool->free = opool->head;
-      opool->head = opool->tail = NULL;
+      opool->bkt_tail->hdr.next = opool->bkt_fini;
+      opool->bkt_fini = opool->bkt_head;
+      opool->bkt_head = opool->bkt_tail = NULL;
     }
 }	
 
 void
 mu_opool_destroy (mu_opool_t *popool)
 {
-  struct mu_opool_bucket *p;
+  union mu_opool_bucket *p;
   if (popool && *popool)
     {
       mu_opool_t opool = *popool;
       mu_opool_clear (opool);
-      for (p = opool->free; p; )
+      for (p = opool->bkt_fini; p; )
 	{
-	  struct mu_opool_bucket *next = p->next;
+	  union mu_opool_bucket *next = p->hdr.next;
 	  free (p);
 	  p = next;
 	}
       free (opool);
+      *popool = NULL;
     }
-  *popool = NULL;
 }
 
 int
@@ -174,13 +207,14 @@ mu_opool_alloc (mu_opool_t opool, size_t size)
     {
       size_t rest;
 
-      if (!opool->head || opool->tail->level == opool->tail->size)
+      if (!opool->bkt_head
+	  || opool->bkt_tail->hdr.level == opool->bkt_tail->hdr.size)
 	if (alloc_pool (opool, opool->bucket_size))
 	  return ENOMEM;
-      rest = opool->tail->size - opool->tail->level;
+      rest = opool->bkt_tail->hdr.size - opool->bkt_tail->hdr.level;
       if (size < rest)
 	rest = size;
-      opool->tail->level += rest;
+      opool->bkt_tail->hdr.level += rest;
       size -= rest;
     }
   return 0;
@@ -217,9 +251,9 @@ size_t
 mu_opool_size (mu_opool_t opool)
 {
   size_t size = 0;
-  struct mu_opool_bucket *p;
-  for (p = opool->head; p; p = p->next)
-    size += p->level;
+  union mu_opool_bucket *p;
+  for (p = opool->bkt_head; p; p = p->hdr.next)
+    size += p->hdr.level;
   return size;
 }
 
@@ -228,14 +262,14 @@ mu_opool_copy (mu_opool_t opool, void *buf, size_t size)
 {
   char *cp = buf;
   size_t total = 0;
-  struct mu_opool_bucket *p;
+  union mu_opool_bucket *p;
   
-  for (p = opool->head; p && total < size; p = p->next)
+  for (p = opool->bkt_head; p && total < size; p = p->hdr.next)
     {
       size_t cpsize = size - total;
-      if (cpsize > p->level)
-	cpsize = p->level;
-      memcpy (cp, p->buf, cpsize);
+      if (cpsize > p->hdr.level)
+	cpsize = p->hdr.level;
+      memcpy (cp, p->hdr.buf, cpsize);
       cp += cpsize;
       total += cpsize;
     }
@@ -249,27 +283,29 @@ mu_opool_coalesce (mu_opool_t opool, size_t *psize)
 
   if (opool->itr_count)
     return MU_ERR_FAILURE;
-  if (opool->head && opool->head->next == NULL)
-    size = opool->head->level;
-  else {
-    struct mu_opool_bucket *bucket;
-    struct mu_opool_bucket *p;
+  if (opool->bkt_head && opool->bkt_head->hdr.next == NULL)
+    size = opool->bkt_head->hdr.level;
+  else
+    {
+      union mu_opool_bucket *bucket;
+      union mu_opool_bucket *p;
 
-    size = mu_opool_size (opool);
+      size = mu_opool_size (opool);
 	
-    bucket = alloc_bucket (opool, size);
-    if (!bucket)
-      return ENOMEM;
-    for (p = opool->head; p; )
-      {
-	struct mu_opool_bucket *next = p->next;
-	memcpy (bucket->buf + bucket->level, p->buf, p->level);
-	bucket->level += p->level;
-	free (p);
-	p = next;
-      }
-    opool->head = opool->tail = bucket;
-  }
+      bucket = alloc_bucket (opool, size);
+      if (!bucket)
+	return ENOMEM;
+      for (p = opool->bkt_head; p; )
+	{
+	  union mu_opool_bucket *next = p->hdr.next;
+	  memcpy (bucket->hdr.buf + bucket->hdr.level, p->hdr.buf,
+		  p->hdr.level);
+	  bucket->hdr.level += p->hdr.level;
+	  free (p);
+	  p = next;
+	}
+      opool->bkt_head = opool->bkt_tail = bucket;
+    }
   if (psize)
     *psize = size;
   return 0;
@@ -278,9 +314,9 @@ mu_opool_coalesce (mu_opool_t opool, size_t *psize)
 void *
 mu_opool_head (mu_opool_t opool, size_t *psize)
 {
-  if (*psize) 
-    *psize = opool->head ? opool->head->level : 0;
-  return opool->head ? opool->head->buf : NULL;
+  if (psize) 
+    *psize = opool->bkt_head ? opool->bkt_head->hdr.level : 0;
+  return opool->bkt_head ? opool->bkt_head->hdr.buf : NULL;
 }
 
 void *
@@ -289,7 +325,64 @@ mu_opool_finish (mu_opool_t opool, size_t *psize)
   if (mu_opool_coalesce (opool, psize))
     return NULL;
   mu_opool_clear (opool);
-  return opool->free->buf;
+  return opool->bkt_fini->hdr.buf;
+}
+
+void *
+mu_opool_detach (mu_opool_t opool, size_t *psize)
+{
+  union mu_opool_bucket *bp;
+  
+  if (mu_opool_coalesce (opool, psize))
+    return NULL;
+  mu_opool_clear (opool);
+
+  bp = opool->bkt_fini;
+  opool->bkt_fini = bp->hdr.next;
+  memmove (bp, bp->hdr.buf, bp->hdr.level);
+  return bp;
+}
+
+void
+mu_opool_free (mu_opool_t pool, void *obj)
+{
+  if (!pool)
+    return;
+  if (!obj)
+    {
+      if (pool->bkt_head)
+	mu_opool_finish (pool, NULL);
+      while (pool->bkt_fini)
+	{
+	  union mu_opool_bucket *next = pool->bkt_fini->hdr.next;
+	  free (pool->bkt_fini);
+	  pool->bkt_fini = next;
+	}
+    }
+  else
+    {
+      union mu_opool_bucket *bucket = pool->bkt_fini,
+	                    **pprev = &pool->bkt_fini;
+      while (bucket)
+	{
+	  if (bucket->hdr.buf == obj)
+	    {
+	      *pprev = bucket->hdr.next;
+	      free (bucket);
+	      return;
+	    }
+	  pprev = &bucket->hdr.next;
+	  bucket = bucket->hdr.next;
+	}
+    }
+}
+
+void *
+mu_opool_dup (mu_opool_t pool, void const *data, size_t size)
+{
+  if (mu_opool_append (pool, data, size))
+    return NULL;
+  return mu_opool_finish (pool, NULL);
 }
 
 int
@@ -314,20 +407,20 @@ mu_opool_union (mu_opool_t *pdst, mu_opool_t *psrc)
   else
     dst = *pdst;
 
-  if (dst->tail)
-    dst->tail->next = src->head;
+  if (dst->bkt_tail)
+    dst->bkt_tail->hdr.next = src->bkt_head;
   else
-    dst->head = src->head;
-  dst->tail = src->tail;
+    dst->bkt_head = src->bkt_head;
+  dst->bkt_tail = src->bkt_tail;
 
-  if (src->free)
+  if (src->bkt_fini)
     {
-      struct mu_opool_bucket *p;
+      union mu_opool_bucket *p;
 
-      for (p = src->free; p->next; p = p->next)
+      for (p = src->bkt_fini; p->hdr.next; p = p->hdr.next)
 	;
-      p->next = dst->free;
-      dst->free = src->free;
+      p->hdr.next = dst->bkt_fini;
+      dst->bkt_fini = src->bkt_fini;
     }
 
   free (src);
@@ -340,14 +433,14 @@ mu_opool_union (mu_opool_t *pdst, mu_opool_t *psrc)
 struct opool_iterator
 {
   mu_opool_t opool;
-  struct mu_opool_bucket *cur;
+  union mu_opool_bucket *cur;
 };
 
 static int
 opitr_first (void *owner)
 {
   struct opool_iterator *itr = owner;
-  itr->cur = itr->opool->head;
+  itr->cur = itr->opool->bkt_head;
   return 0;
 }
 
@@ -357,7 +450,7 @@ opitr_next (void *owner)
   struct opool_iterator *itr = owner;
   if (itr->cur)
     {
-      itr->cur = itr->cur->next;
+      itr->cur = itr->cur->hdr.next;
       return 0;
     }
   return EINVAL;
@@ -370,9 +463,9 @@ opitr_getitem (void *owner, void **pret, const void **pkey)
   if (!itr->cur)
     return MU_ERR_NOENT;
 
-  *pret = itr->cur->buf;
+  *pret = itr->cur->hdr.buf;
   if (pkey)
-    *(size_t*) pkey = itr->cur->level;
+    *(size_t*) pkey = itr->cur->hdr.level;
   return 0;
 }
 
@@ -387,7 +480,7 @@ static int
 opitr_delitem (void *owner, void *item)
 {
   struct opool_iterator *itr = owner;
-  return (itr->cur && itr->cur->buf == item) ? 
+  return (itr->cur && itr->cur->hdr.buf == item) ? 
      MU_ITR_DELITEM_NEXT : MU_ITR_DELITEM_NOTHING;
 }
 
@@ -420,6 +513,31 @@ opitr_data_dup (void **ptr, void *owner)
   return 0;
 }
 
+static int
+opitr_itrctl (void *owner, enum mu_itrctl_req req, void *arg)
+{
+  struct opool_iterator *itr = owner;
+  switch (req)
+    {
+    case mu_itrctl_count:
+      if (!arg)
+	return EINVAL;
+      else
+	{
+	  size_t n = 0;
+	  union mu_opool_bucket *p;
+	  for (p = itr->opool->bkt_head; p; p = p->hdr.next)
+	    n++;
+	  *(size_t*)arg = n;
+	}
+      break;
+
+    default:
+      return ENOSYS;
+    }
+  return 0;
+}
+	  
 int
 mu_opool_get_iterator (mu_opool_t opool, mu_iterator_t *piterator)
 {
@@ -434,7 +552,7 @@ mu_opool_get_iterator (mu_opool_t opool, mu_iterator_t *piterator)
   if (!itr)
     return ENOMEM;
   itr->opool = opool;
-  itr->cur = opool->head;
+  itr->cur = opool->bkt_head;
   
   status = mu_iterator_create (&iterator, itr);
   if (status)
@@ -450,7 +568,8 @@ mu_opool_get_iterator (mu_opool_t opool, mu_iterator_t *piterator)
   mu_iterator_set_delitem (iterator, opitr_delitem);
   mu_iterator_set_destroy (iterator, opitr_destroy);
   mu_iterator_set_dup (iterator, opitr_data_dup);
-
+  mu_iterator_set_itrctl (iterator, opitr_itrctl);
+  
   opool->itr_count++;
 
   *piterator = iterator;
