@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2004-2005, 2007-2008, 2010-2012, 2014-2015
+   Copyright (C) 1999-2001, 2004-2005, 2007-2008, 2010-2012, 2014-2017
    Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -205,6 +205,11 @@ list_itrctl (void *owner, enum mu_itrctl_req req, void *arg)
 	itr->backwards = !!*(int*)arg;
       break;
       
+    case mu_itrctl_count:
+      if (!arg)
+	return EINVAL;
+      return mu_list_count (itr->list, arg);
+      
     default:
       return ENOSYS;
     }
