@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2004-2012, 2014-2015 Free Software
+   Copyright (C) 1999-2001, 2004-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -446,10 +446,10 @@ mu_folder_delete (mu_folder_t folder, const char *name)
 	 undesired results.  For example, a MH mailbox can hold another
 	 mailboxes, i.e. be a folder itself.  Removing it blindly would 
 	 result in removing these mailboxes as well, which is clearly not 
-	 indended.
+	 intended.
 
-	 To solve this folder and mailbox delete methods are tightly paired,
-	 but without looking into each-others internal mechanisms. */
+	 To solve this, both folder and mailbox delete methods are tightly
+	 paired, but without looking into each-others internal mechanisms. */
       mu_mailbox_t mbox;
       rc = mu_mailbox_create_at (&mbox, folder, name);
       if (rc == 0)
