@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2006-2007, 2009-2012, 2014-2015 Free
+   Copyright (C) 1999-2001, 2006-2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -289,10 +289,13 @@ SCM_DEFINE_PUBLIC (scm_mu_message_set_header, "mu-message-set-header", 3, 1, 0,
 
   hdr_c = scm_to_locale_string (header);
   val_c = scm_to_locale_string (value);
-  status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+  if (repl)
+    status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+  else
+    status = mu_header_append (hdr, hdr_c, val_c);
   free (hdr_c);
   free (val_c);
-  
+
   if (status)
     mu_scm_error (FUNC_NAME, status,
 		  "Cannot set header \"~A: ~A\" in message ~A",
@@ -602,7 +605,10 @@ SCM_DEFINE_PUBLIC (scm_mu_message_set_header_fields, "mu-message-set-header-fiel
       SCM_ASSERT (scm_is_string (cdr), cdr, SCM_ARGn, FUNC_NAME);
       hdr_c = scm_to_locale_string (car);
       val_c = scm_to_locale_string (cdr);
-      status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+      if (repl)
+	status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+      else
+	status = mu_header_append (hdr, hdr_c, val_c);
       free (hdr_c);
       free (val_c);
       if (status)
@@ -1068,7 +1074,7 @@ SCM_DEFINE_PUBLIC (scm_mu_message_send, "mu-message-send", 1, 3, 0,
   status = mu_mailer_create (&mailer_c, mailer_name);
   free (mailer_name);
   if (status)
-    mu_scm_error (FUNC_NAME, status, "Cannot get create mailer", SCM_BOOL_F);
+    mu_scm_error (FUNC_NAME, status, "Cannot create mailer", SCM_BOOL_F);
 
   if (scm_to_int (MU_SCM_SYMBOL_VALUE ("mu-debug")))
     {
