@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2016 Free
+   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -33,7 +33,7 @@ sig_handler (int signo)
   switch (signo)
     {
     case SIGINT:
-      if (mailvar_get (NULL, "quit", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("quit"))
 	exit (0);
       _interrupted++;
       break;
@@ -93,7 +93,8 @@ ml_readline_init ()
 
 #ifdef WITH_READLINE
   rl_readline_name = "mail";
-  rl_attempted_completion_function = (CPPFunction*)ml_command_completion;
+  rl_attempted_completion_function =
+    (rl_completion_func_t*) ml_command_completion;
   rl_getc_function = ml_getc;
 #endif
 #ifdef HAVE_SIGACTION
