@@ -210,12 +210,14 @@ struct mh_whatnow_env     /* whatnow shell environment */
   char *file;             /* The file being processed */
   char *msg;              /* File name of the original message (if any) */
   char *draftfile;        /* File to preserve the draft into */
-  const char *editor;
+  const char *editor;     /* Default editor */
   char *prompt;
   char *anno_field;       /* Annotate field to be used */
   mu_list_t anno_list;    /* List of messages (mu_message_t) to annotate */
   mu_mailbox_t mbox;
   int nowhatnowproc;
+  int reedit:1;           /* Set if the editor was already invoked */
+  char *last_ed;          /* Last used editor */
 };
 
 #define DISP_QUIT 0
@@ -300,12 +302,13 @@ int mh_message_number (mu_message_t msg, size_t *pnum);
 mu_mailbox_t mh_open_folder (const char *folder, int flags);
 
 void mh_msgset_parse (mu_msgset_t *msgset, mu_mailbox_t mbox, 
-		      int argc, char **argv, char *def);
+		      int argc, char **argv, const char *def);
 void mh_msgset_parse_string (mu_msgset_t *msgset, mu_mailbox_t mbox, 
-			     const char *string, char *def);
-void mh_msgset_first_current (mu_mailbox_t mbox, mu_msgset_t msgset);
-size_t mh_msgset_first (mu_msgset_t msgset);
-size_t mh_msgset_first_uid (mu_msgset_t msgset);
+			     const char *string, const char *def);
+#define RET_MSGNO 0
+#define RET_UID   1
+size_t mh_msgset_first (mu_msgset_t msgset, int uid);
+size_t mh_msgset_last  (mu_msgset_t msgset, int uid);
 int mh_msgset_single_message (mu_msgset_t msgset);
 
 #define NAME_ANY    0
@@ -386,5 +389,6 @@ char *mh_safe_make_file_name (const char *dir, const char *file);
 void mh_mailbox_get_cur (mu_mailbox_t mbox, size_t *pcur);
 void mh_mailbox_set_cur (mu_mailbox_t mbox, size_t cur);
 
-void mh_whatnow_env_from_environ (struct mh_whatnow_env *wh);
+void mh_whatnow_env_from_environ_early (struct mh_whatnow_env *wh);
+void mh_whatnow_env_from_environ_late (struct mh_whatnow_env *wh);
 void mh_whatnow_env_to_environ (struct mh_whatnow_env *wh);
