@@ -69,7 +69,7 @@
 #line 1 "mh_fmtgram.y"
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2004, 2007, 2009-2012, 2014-2015 Free
+   Copyright (C) 1999-2002, 2004, 2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -2234,7 +2234,7 @@ mh_format_parse (char *format_str, mh_format_t *fmt)
   if (p)
     fmt_yydebug = 1;
   start = curp = format_str;
-  mu_opool_create (&tokpool, 1);
+  mu_opool_create (&tokpool, MU_OPOOL_ENOMEMABRT);
   format.prog = NULL;
   format.progsize = 0;
   pc = 0;
