@@ -208,7 +208,7 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
       break;  
     }
 
-  msgno = mh_msgset_first (msgset);
+  msgno = mh_msgset_first (msgset, RET_MSGNO);
   rc = mu_mailbox_get_message (mbox, msgno, &msg);
   if (rc)
     {
@@ -247,7 +247,7 @@ make_draft (mu_mailbox_t mbox, int disp, struct mh_whatnow_env *wh)
 	  mu_message_create_copy (&tmp_msg, msg);
 	  mu_message_get_header (tmp_msg, &hdr);
 	  text = mu_opool_finish (fcc_pool, NULL);
-	  mu_header_set_value (hdr, MU_HEADER_FCC, text, 0);
+	  mu_header_set_value (hdr, MU_HEADER_FCC, text, 1);
 	  mh_format (&format, tmp_msg, msgno, width, &buf);
 	  mu_message_destroy (&tmp_msg, NULL);
 	}
@@ -294,12 +294,15 @@ main (int argc, char **argv)
 {
   int rc;
 
-  draftfolder = mh_global_profile_get ("Draft-Folder", NULL);
-  whatnowproc = mh_global_profile_get ("whatnowproc", NULL);
-
   mh_getopt (&argc, &argv, options, MH_GETOPT_DEFAULT_FOLDER,
 	     args_doc, prog_doc, NULL);
 
+  if (!draftfolder)
+    draftfolder = mh_global_profile_get ("Draft-Folder", NULL);
+  if (!whatnowproc)
+    whatnowproc = mh_global_profile_get ("whatnowproc", NULL);
+
+  
   if (!format_str)
     format_str = default_format_str;
 
